// FTP.h: interface for the TFTP class.
//


#pragma once

#undef DECL
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif

#include "Socket.h"
#include "TcpPort.h"

class DECL TFTP 
{
public:
   TFTP(CString sURL, int iPort, CString sUserName, CString sPassword);
   ~TFTP();
   typedef enum 
   {
     ftpSuccess=0, ftpCouldNotConnect=1, ftpCouldNotLogin=2, ftpFileDoesNotExist=3, 
     ftpCommandDenied=4, ftpCouldNotRenamePartialFile=5, ftpCouldNotEstablishDataChannel=6,
     ftpCouldNotConnectToDataChannel=7, ftpCouldNotStoreToDestination=8, ftpCouldNotOpenSource=9,
     ftpCouldNotCreateDestination=10, ftpCouldNotReadFromSource=11, ftpCouldNotWriteToDestination=12,
     ftpFailureWhileSendingData=13, ftpServerDidNotReportSuccess=14, ftpOperationWasCancelled=15, 
     ftpNotConnected=16, ftpInvalidHandle=17, ftpLastResult
   } TResult;
   TResult Connect();
   TResult Put(CString SourceFile, CString DestFile, bool AutoResume);
   TResult Get(CString SourceFile, CString DestFile, bool AutoResume);
   TResult Store(TData& Buffer, CString DestFile);
   TResult Append(TData& Buffer, CString DestFile);
   void Close();
   void SetFileType(CString sFileType)  { FileType = sFileType;               };
   DWORD GetBytesSent()                 { return nBytesSent;                  };
   DWORD GetBytesRetrieved()            { return nBytesRetrieved;             };
   void Cancel()                        { SetEvent(hCancelEvent);             };
   bool IsConnected()                   { return pCommandPort && pCommandPort->IsOpen(); };
   static LPCTSTR GetErrorStr(TResult Result);
   CString ReservedBasicAPI;
public:
   // Various FTP Commands that can be issued once a Connect() has been made
   bool ABOR()                                 { return Command(_T("ABOR"));                };
   bool ACCT(CString Parm)                     { return Command(_T("ACCT"), Parm);          };
   bool CDUP()                                 { return Command(_T("CDUP"));                };
   bool CWD(CString Parm)                      { return Command(_T("CWD"),  Parm);          };
   bool DELE(CString Parm)                     { return Command(_T("DELE"), Parm);          };
   bool HELP(CString Parm, CString& Reply)     { return Command(_T("HELP"), Parm,   Reply); };
   bool LIST(CString Parm, CString& Reply)     { return GETCMD(_T("LIST"),  Parm,   Reply); };
   bool MKD(CString Parm)                      { return Command(_T("MKD"),  Parm);          };
   bool NLST(CString Parm, CString& Reply)     { return GETCMD(_T("NLST"),  Parm,   Reply); };
   bool PWD(CString& Reply)                    { return Command(_T("PWD"),  _T(""), Reply); };
   bool QUIT()                                 { return Command(_T("QUIT"));                };
   bool REIN()                                 { return Command(_T("REIN"));                };
   bool REN(CString OldName, CString NewName)  { return RNFR(OldName) & RNTO(NewName);      };
   bool REST(DWORD Pos)                        { return Command(_T("REST"), Pos);           };
   bool RMD(CString Parm)                      { return Command(_T("RMD"),  Parm);          };
   bool SITE(CString Parm, CString& Reply)     { return Command(_T("SITE"), Parm,   Reply); };
   bool SIZE(CString DestFile, DWORD& Size);
   bool SMNT(CString Parm)                     { return Command(_T("SMNT"), Parm);          };
   bool STAT(CString Parm, CString& Reply)     { return Command(_T("STAT"), Parm,   Reply); };
   bool SYST(CString& Reply)                   { return Command(_T("SYST"), _T(""), Reply); };
protected:
   TSocket CommandSocket;
   CTcpPort* pCommandPort;
   CString URL;
   CString UserName;
   CString Password;
   int Port;
   CString FileType;
protected:
   bool Command(CString Cmd, CString Parm, CString& Reply);
   bool Command(CString Cmd, CString Parm);
   bool Command(CString Cmd) { return Command(Cmd, _T("")); };
   bool Command(CString Cmd, DWORD dwNum);
   static void SetTimeout(CTcpPort& port, int iWaitMS);
   // Various FTP Commands
   bool USER(CString UserName) { return Command(_T("USER"), UserName); };
   bool PASS(CString Password) { return Command(_T("PASS"), Password); };
   bool TYPE(CString FileType) { return Command(_T("TYPE"), FileType); };
   bool PASV(CString& DataIP, DWORD& DataPort);
   bool STOR(CString DestFile) { return Command(_T("STOR"), DestFile); };
   bool RETR(CString SrcFile)  { return Command(_T("RETR"), SrcFile);  };
   bool GETCMD(CString Cmd, CString Parm, CString& Reply);
   bool RNFR(CString SrcFile)  { return Command(_T("RNFR"), SrcFile);  };
   bool RNTO(CString DestFile) { return Command(_T("RNTO"), DestFile); };
   TResult EstablishDataChannel(TSocket& DataSocket);
   static DWORD GetFileLen(LPCTSTR strPath);
   bool LineInput(CString& str);
protected:
   HANDLE hCancelEvent;
   DWORD nBytesSent;
   DWORD nBytesRetrieved;
};

// This section may be pasted in to a Basic program and used as an interface to the FTP API
#if 0
' This is the FTP API ver 1.0 which is built in to Xpert2 builds 3.3.0.11 and later
' * Send files to an FTP Server with Ftp_Put
' * Get files from an FTP Server with Ftp_Get
' * Append strings to a file on an FTP Server with Ftp_Append
' * Store strings to a file on an FTP Server with Ftp_Store
' * Transfers can be resumed after an error
' * Retrieve the list of files on an FTP Server with Ftp_LIST
' * Supports many other FTP commands
' * Only supports simple authentication at this time

' Parameter that begin with "s" are strings, "i" are integers, and "b" are boolean

' Please note that file paths on an FTP Server use Unix syntax and hence forward slashes
' (ex: "/home/public/myfile.txt") and are case sensitive, while file paths referencing the 
' Xpert logger use the Microsoft syntax and hence back slashes (ex: "\Flash Disk\SiteImage.jpg")

' Result codes used by the FTP API
Const ftpSuccess=0
Const ftpCouldNotConnect=1
Const ftpCouldNotLogin=2
Const ftpFileDoesNotExist=3
Const ftpCommandDenied=4
Const ftpCouldNotRenamePartialFile=5
Const ftpCouldNotEstablishDataChannel=6
Const ftpCouldNotConnectToDataChannel=7
Const ftpCouldNotStoreToDestination=8
Const ftpCouldNotOpenSource=9
Const ftpCouldNotCreateDestination=10
Const ftpCouldNotReadFromSource=11
Const ftpCouldNotWriteToDestination=12
Const ftpFailureWhileSendingData=13
Const ftpServerDidNotReportSuccess=14
Const ftpOperationWasCancelled=15
Const ftpNotConnected=16
Const ftpInvalidHandle=17

' This function intializes a FTP API session
Declare Function Ftp_Open Lib "\Windows\Utils.dll" (sURL, iPort, sUserName, sPassword) As Integer
' Parameters
'    sURL
'       Address of FTP Server
'    iPort
'       Port number of FTP Server (typically 21)
'    sUserName
'       Used for simple authentication
'    sPassword
'       Used for simple authentication
' Return Value
'    A handle value that must be passed to the other Ftp API functions

' This function terminates a Ftp API session
Declare Function Ftp_Close Lib "\Windows\Utils.dll" (iHandle) As Integer
' Parameters
'    iHandle
'       value returned by Ftp_Open
' Return Value
'    1 indicates success, 0 failure

' This function initiates a connection to the FTP Server
Declare Function Ftp_Connect Lib "\Windows\Utils.dll" (iHandle) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
' Return Value
'    An error code (0=Success)

' This function sets the file type for an FTP transfer (the default if the function is not called is a binary image "I")
Declare Sub Ftp_SetFileType Lib "\Windows\Utils.dll" (iHandle, sFileType)
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sFileType
'       An FTP file type such as "I" for an image (binary data) or "A" for ASCII text
'          "An" - ASCII, where n may be omitted, N, T, or C (see FTP TYPE command doc)
'          "En" - EBCDIC, where n may be omitted, N, T, or C (see FTP TYPE command doc)
'          "I"  - IMAGE (binary data)
'          "Ln" - local format, where n is the number of bits per byte
' Return Value
'    An error code (0=Success)

' Returns the number of bytes sent so far
Declare Function Ftp_GetBytesSent Lib "\Windows\Utils.dll" (iHandle) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
' Return Value
'    The number of bytes sent

' Returns the number of bytes retrieved so far
Declare Function Ftp_GetBytesRetrieved Lib "\Windows\Utils.dll" (iHandle) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
' Return Value
'    The number of bytes Retrieved

' Cancels an FTP operation in progress
Declare Sub Ftp_Cancel Lib "\Windows\Utils.dll" (iHandle)
' Parameters
'    iHandle
'       Value returned by Ftp_Open


' Returns the connected status of a session
Declare Function Ftp_IsConnected Lib "\Windows\Utils.dll" (iHandle) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
' Return Value
'    0: Not connected, 1: Connected

' Converts an error code in to a string
Declare Function Ftp_GetErrorStr Lib "\Windows\Utils.dll" (iError) As String
' Parameters
'    iError
'       Error code returns by Ftp_Connect, Ftp_Put, Ftp_Get, Ftp_Store, or Ftp_Append
' Return Value
'    A string representing the error value

' Transfers a file to the FTP Server with optional support for resume
Declare Function Ftp_Put Lib "\Windows\Utils.dll" (iHandle, sSourceFile, sDestFile, bAutoResume) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sSourceFile
'       File to send
'    sDestFile
'       What to call it on the server
'    bAutoResume
'       Whether to automatically resume a previous failed transfer
' Return Value
'    An error code (0=Success)

' Retrieves a file from the FTP Server with optional support for resume
Declare Function Ftp_Get Lib "\Windows\Utils.dll" (iHandle, sSourceFile, sDestFile, bAutoResume) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sSourceFile
'       File to retrieve
'    sDestFile
'       Where to store it on the Xpert
'    bAutoResume
'       Whether to automatically resume a previous failed transfer
' Return Value
'    An error code (0=Success)

' Stores a string to a file on the FTP Server
Declare Function Ftp_Store Lib "\Windows\Utils.dll" (iHandle, sBuffer, sDestFile) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sBuffer
'       String to store
'    sDestFile
'       Where to store it on the server (will overwrite an existing file)
' Return Value
'    An error code (0=Success)

' Appends a string to a file on the FTP Server
Declare Function Ftp_Append Lib "\Windows\Utils.dll" (iHandle, sBuffer, sDestFile) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sBuffer
'       String to append
'    sDestFile
'       What to append it to on the server (must already exist)
' Return Value
'    An error code (0=Success)

'
' ---------------------------------------------------------
' The following commands are "RAW" FTP commands that
' may be issued once a connection is established. The
' results are either simple booleans (1=Success, 0=Fail)
' or it may be the reply as provided by the FTP server.
' This means there may be ftp codes in the result. These
' codes are 3 digit ASCII numbers. Codes from 100-399
' generally indicate a successfully operation, while codes
' from 400-599 generally indicate some sort of failure.
' ---------------------------------------------------------
'

' Aborts a file transfer in progress
Declare Function Ftp_ABOR Lib "\Windows\Utils.dll" (iHandle) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
' Return Value
'    1: Success, 0: Failure

' Sends account information (beyond the user name and password)
Declare Function Ftp_ACCT Lib "\Windows\Utils.dll" (iHandle, sParm) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sParm
'       The data to be passed to the command
' Return Value
'    1: Success, 0: Failure

' Changes the present working directory to the parent directory
Declare Function Ftp_CDUP Lib "\Windows\Utils.dll" (iHandle) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
' Return Value
'    1: Success, 0: Failure

' Changes the present working directory to the parameter
Declare Function Ftp_CWD Lib "\Windows\Utils.dll" (iHandle, sParm) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sParm
'       The data to be passed to the command
' Return Value
'    1: Success, 0: Failure

' Deletes the specified file passed in the parameter
Declare Function Ftp_DELE Lib "\Windows\Utils.dll" (iHandle, sParm) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sParm
'       The data to be passed to the command
' Return Value
'    1: Success, 0: Failure

' Returns help information for the command passed in the parameter
' or if the paramater is blank, returns the list of commands
Declare Function Ftp_HELP Lib "\Windows\Utils.dll" (iHandle, sParm) As String
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sParm
'       The data to be passed to the command
' Return Value
'    A string with a 3 digit FTP result code at the beginning that
'    may contain multiple <CR><LF> seperated lines

' Returns a file list based on the wildcard path passed in the parameter
' the result will vary depending on the server but is typically based on 
' either Unix "ls" command or the Microsoft "dir" command and may include
' verbose file times, owner information, and attributes
Declare Function Ftp_LIST Lib "\Windows\Utils.dll" (iHandle, sParm) As String
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sParm
'       The data to be passed to the command
' Return Value
'    A string with a 3 digit FTP result code at the beginning that
'    may contain multiple <CR><LF> seperated lines

' Makes a new directory as specified in the parameter
Declare Function Ftp_MKD Lib "\Windows\Utils.dll" (iHandle, sParm) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sParm
'       The data to be passed to the command
' Return Value
'    1: Success, 0: Failure

' Returns a file list based on the wildcard path passed in the parameter.
' The list is abbreviated compared to the LIST command and typically
' contains a list of files seperated by <CR><LF>
Declare Function Ftp_NLST Lib "\Windows\Utils.dll" (iHandle, sParm) As String
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sParm
'       The data to be passed to the command
' Return Value
'    A string with a 3 digit FTP result code at the beginning that
'    may contain multiple <CR><LF> seperated lines

Declare Function Ftp_PWD Lib "\Windows\Utils.dll" (iHandle) As String

' Terminates a session
Declare Function Ftp_QUIT Lib "\Windows\Utils.dll" (iHandle) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
' Return Value
'    1: Success, 0: Failure

' Reinitializes a session
Declare Function Ftp_REIN Lib "\Windows\Utils.dll" (iHandle) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
' Return Value
'    1: Success, 0: Failure

' Renames a file on the FTP Server to a new name
Declare Function Ftp_REN Lib "\Windows\Utils.dll" (iHandle, sOldName, sNewName) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sOldName
'       The file to be renamed
'    sNewName
'       The new name to give the file
' Return Value
'    1: Success, 0: Failure

' Sets the file pointer for the next file transfer operation
Declare Function Ftp_REST Lib "\Windows\Utils.dll" (iHandle, iPos) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    iPos
'       Byte number to resume the next file transfer at
' Return Value
'    1: Success, 0: Failure

' Removes the directory specified by the parameter
Declare Function Ftp_RMD Lib "\Windows\Utils.dll" (iHandle, sParm) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sParm
'       The data to be passed to the command
' Return Value
'    1: Success, 0: Failure

' Performs a site specific command, one command often supported is the Unix 
' CHMOD command for changing file rights. The list may be retrieved by 

' passing "HELP" as the parameter.
Declare Function Ftp_SITE Lib "\Windows\Utils.dll" (iHandle, sParm) As String
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sParm
'       The data to be passed to the command
' Return Value
'    A string with a 3 digit FTP result code at the beginning that
'    may contain multiple <CR><LF> seperated lines

' Retrieves the size of the file passed in the parameter (in bytes)
Declare Function Ftp_SIZE Lib "\Windows\Utils.dll" (iHandle, sParm) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sParm
'       The data to be passed to the command
' Return Value
'    1: Success, 0: Failure

' Mounts the device passed in the paramater (not often supported for security
' reasons)
Declare Function Ftp_SMNT Lib "\Windows\Utils.dll" (iHandle, sParm) As Integer
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sParm
'       The data to be passed to the command
' Return Value
'    1: Success, 0: Failure

' Returns server status when the passed paramater is null, otherwise
' acts like the LIST commands and displays file matching the path
' passed in the parameter
Declare Function Ftp_STAT Lib "\Windows\Utils.dll" (iHandle, sParm) As String
' Parameters
'    iHandle
'       Value returned by Ftp_Open
'    sParm
'       The data to be passed to the command
' Return Value
'    1: Success, 0: Failure

' Returns a string identifying the system hosting the FTP Server
' (ex: "215 UNIX Type: L8")
Declare Function Ftp_SYST Lib "\Windows\Utils.dll" (iHandle) As String
' Parameters
'    iHandle
'       Value returned by Ftp_Open
' Return Value
'    A string with a 3 digit FTP result code at the beginning that
'    may contain multiple <CR><LF> seperated lines


' A simple program to transfer a file might look like this:
'
' Handle = Ftp_Open("ftp.myserver.com", 21, "username", "password")
' If Handle Then
'    Result = Ftp_Connect(Handle)
'   If Result <> 0 Then
'      ErrorMsg "FTP Connect failed with: " & Ftp_GetErrorStr(Result)
'   Else
'      Result = Ftp_Put(Handle, "\Flash Disk\image.jpg", "/public/image.jpg", True)
'      If Result <> 0 Then
'         ErrorMsg "FTP Put failed with: " & Ftp_GetErrorStr(Result)
'      End If
'   End If
'   Result = Ftp_Close(Handle)
'
'
#endif

// FTP API for Basic
extern "C" DECL DWORD Ftp_Open(LPCTSTR sURL, int iPort, LPCTSTR sUserName, LPCTSTR sPassword);
extern "C" DECL DWORD Ftp_Connect(DWORD dwHandle);
extern "C" DECL void Ftp_SetFileType(DWORD dwHandle, LPCTSTR FileType);
extern "C" DECL DWORD Ftp_GetBytesSent(DWORD dwHandle);
extern "C" DECL DWORD Ftp_GetBytesRetrieved(DWORD dwHandle);
extern "C" DECL void Ftp_Cancel(DWORD dwHandle);
extern "C" DECL BOOL Ftp_IsConnected(DWORD dwHandle);
extern "C" DECL LPCTSTR Ftp_GetErrorStr(DWORD Error);
extern "C" DECL DWORD Ftp_Put(DWORD dwHandle, LPCTSTR SourceFile, LPCTSTR DestFile, BOOL AutoResume);
extern "C" DECL DWORD Ftp_Get(DWORD dwHandle, LPCTSTR SourceFile, LPCTSTR DestFile, BOOL AutoResume);
extern "C" DECL DWORD Ftp_Store(DWORD dwHandle, LPCTSTR Buffer, LPCTSTR DestFile);
extern "C" DECL DWORD Ftp_Append(DWORD dwHandle, LPCTSTR Buffer, LPCTSTR DestFile);
extern "C" DECL BOOL Ftp_Close(DWORD dwHandle);

// Miscellaneous FTP commands that may be issued
extern "C" DECL BOOL Ftp_ABOR(DWORD dwHandle);
extern "C" DECL BOOL Ftp_ACCT(DWORD dwHandle, LPCTSTR Parm);
extern "C" DECL BOOL Ftp_CDUP(DWORD dwHandle);
extern "C" DECL BOOL Ftp_CWD(DWORD dwHandle, LPCTSTR Parm);
extern "C" DECL BOOL Ftp_DELE(DWORD dwHandle, LPCTSTR Parm);
extern "C" DECL LPCTSTR Ftp_HELP(DWORD dwHandle, LPCTSTR Parm);
extern "C" DECL LPCTSTR Ftp_LIST(DWORD dwHandle, LPCTSTR Parm);
extern "C" DECL BOOL Ftp_MKD(DWORD dwHandle, LPCTSTR Parm);
extern "C" DECL LPCTSTR Ftp_NLST(DWORD dwHandle, LPCTSTR Parm);
extern "C" DECL LPCTSTR Ftp_PWD(DWORD dwHandle);
extern "C" DECL BOOL Ftp_QUIT(DWORD dwHandle);
extern "C" DECL BOOL Ftp_REIN(DWORD dwHandle);
extern "C" DECL BOOL Ftp_REN(DWORD dwHandle, LPCTSTR OldName, LPCTSTR NewName);
extern "C" DECL BOOL Ftp_REST(DWORD dwHandle, DWORD Pos);
extern "C" DECL BOOL Ftp_RMD(DWORD dwHandle, LPCTSTR Parm);
extern "C" DECL LPCTSTR Ftp_SITE(DWORD dwHandle, LPCTSTR Parm);
extern "C" DECL DWORD Ftp_SIZE(DWORD dwHandle, LPCTSTR Parm);
extern "C" DECL BOOL Ftp_SMNT(DWORD dwHandle, LPCTSTR Parm);
extern "C" DECL LPCTSTR Ftp_STAT(DWORD dwHandle, LPCTSTR Parm);
extern "C" DECL LPCTSTR Ftp_SYST(DWORD dwHandle);
